@@ -1,3 +1,4 @@
+use kwap_common::GetSize;
 use tinyvec::ArrayVec;
 
 use crate::*;
@@ -42,7 +43,7 @@ pub trait TryIntoBytes {
   ///
   /// let bytes: Vec<u8> = vec_message.try_into_bytes().unwrap();
   /// ```
-  fn try_into_bytes<C: Collection<u8>>(self) -> Result<C, Self::Error>
+  fn try_into_bytes<C: Array<u8>>(self) -> Result<C, Self::Error>
     where for<'a> &'a C: IntoIterator<Item = &'a u8>;
 }
 
@@ -53,14 +54,14 @@ pub enum MessageToBytesError {
   TooLong { capacity: usize, size: usize },
 }
 
-impl<P: Collection<u8>, O: Collection<u8>, Os: Collection<Opt<O>>> TryIntoBytes for Message<P, O, Os>
+impl<P: Array<u8>, O: Array<u8>, Os: Array<Opt<O>>> TryIntoBytes for Message<P, O, Os>
   where for<'b> &'b P: IntoIterator<Item = &'b u8>,
         for<'b> &'b O: IntoIterator<Item = &'b u8>,
         for<'b> &'b Os: IntoIterator<Item = &'b Opt<O>>
 {
   type Error = MessageToBytesError;
 
-  fn try_into_bytes<C: Collection<u8>>(self) -> Result<C, Self::Error>
+  fn try_into_bytes<C: Array<u8>>(self) -> Result<C, Self::Error>
     where for<'a> &'a C: IntoIterator<Item = &'a u8>
   {
     let mut bytes = C::reserve(self.get_size());
